<?php

declare(strict_types=1);

namespace Drupal\addevent\Api;

use Drupal\addevent\Contract\AddEventCalendarApiInterface;

/**
 * Define the AddEvent Calendar API class.
 */
class AddEventCalendarApi extends BaseApi implements AddEventCalendarApiInterface {

  /**
   * {@inheritDoc}
   */
  protected const ENDPOINT_BASE_URL = 'https://www.addevent.com/api/v1/me/calendars';

  /**
   * {@inheritDoc}
   */
  public function listCalendars(): array {
    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'list'
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function createCalendar(
    string $title,
    array $query = []
  ): array {
    $query['title'] = $title;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'create',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function saveCalendar(
    string $title,
    string $calendar_id,
    array $query = []
  ): array {
    $query['title'] = $title;
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'save',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function deleteCalendar(
    string $calendar_id
  ): array {
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'delete',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function listCalendarEvents(
    string $calendar_id,
    array $query = []
  ): array {
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'events/list',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function createCalendarEvent(
    string $title,
    string $calendar_id,
    array $query = []
  ): array {
    $query['title'] = $title;
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'events/create',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function saveCalendarEvent(
    string $title,
    string $event_id,
    array $query = []
  ): array {
    $query['title'] = $title;
    $query['event_id'] = $event_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'events/save',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function deleteCalendarEvent(
    string $event_id
  ): array {
    $query['event_id'] = $event_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'events/delete',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function listCalenderSubscribers(
    string $calendar_id
  ): array {
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'subscribers/list',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function viewCalenderSubscriber(
    string $user_id,
    string $calendar_id
  ): array {
    $query['user_id'] = $user_id;
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'subscribers/subscriber',
        query: $query
      )
    );
  }

  /**
   * {@inheritDoc}
   */
  public function deleteCalenderSubscriber(
    string $user_id,
    string $calendar_id
  ): array {
    $query['user_id'] = $user_id;
    $query['calendar_id'] = $calendar_id;

    return $this->parseResponseContent(
      $this->makeRequest(
        path: 'subscribers/subscriber/delete',
        query: $query
      )
    );
  }
}
